#!/usr/bin/python
# -*- coding: utf-8 -*-
import sys
import time
import struct

import SiTCP


##########################################################
def main(argv):
	
	try:
		sitcp = SiTCP.UDP('192.168.10.128')		# RBCP open
		tcp = SiTCP.TCP(0.1, '192.168.10.128')	# TCP/IP connect

		configure_device(sitcp)
		acquire_data(sitcp, tcp, 3.0)	# data acquisition for 3.0 seconds

		sitcp.close()	# RBCP close
		tcp.close()		# TCP/IP disconnect
	except Exception as ex:
		print (ex)


##########################################################
def acquire_data(sitcp, tcp, acq_sec):

	HIST_BINS = 4096	# 32bit, 4096bins
#	HIST_BINS = 8192	# 32bit, 8192bins

	histdata = [ 0 for n in range(HIST_BINS) ]
	rlt_sec = -1.0

	# start acquisition
	sitcp.write2byte(0xb4000014, 0)
	sitcp.write2byte(0xb4000014, 1)

	# acquisition loop
	while acq_sec > rlt_sec:
		# read status
		icr = sitcp.read4byte(0xb400022c)	# input count rate of CH1
		tcr = sitcp.read4byte(0xb4000230)	# throughput count rate of CH1
		rlt = sitcp.read6byte(0xb400001c)	# real time

		# convert real time(digits to seconds)
		rlt_sec = float(rlt) / 100000000.0

		print ("[%.3f/%.3f], input count rate = %d, throughput count rate = %d." % (rlt_sec, acq_sec, icr, tcr))

		# read histogram
		sitcp.write2byte(0xb400004a, 0)		# request histogram of CH1
		# time.sleep(0.01)					# enable this statement if you receive an error receiving histogram data.
		rxdata, rxbytes = tcp.read(HIST_BINS * 4)	# spectrum read

		for i in range(0, rxbytes, 4):	# to 32bit value
			histdata[int(i / 4)], = struct.unpack("!I", rxdata[i:i+4])
		
		time.sleep(0.2)		# wait, 200 milliseconds
	
	# stop acquisition
	sitcp.write2byte(0xb4000014, 0)

	# save histogram data to CSV file
	with open('./histogram.csv', 'w') as csv:
		# write header
		text  = '[header]\n'
		text += 'Real time(s),%d\n' % (int(rlt_sec))
		text += 'Input rate(cps),%d\n' % (icr)
		text += 'Throughput rate(cps),%d\n' % (tcr)
		csv.writelines(text)

		# write data
		text = '[data]\n'
		for i in range(len(histdata)):
			text += '%d\n' % (histdata[i])
		csv.writelines(text)
	
	return


##########################################################
def configure_device(sitcp):

	# CH register: CH1 only
	sitcp.write2byte(0xb4000200, 1)		# analog coarse gain: x5
	sitcp.write2byte(0xb4000202, 1)		# ADC gain: 4096 bins
	sitcp.write2byte(0xb4000208, 80)	# slow rise time: 800nsec(10ns/digit)
	sitcp.write2byte(0xb400020a, 100)	# slow peaking time: 1usec(10ns/digit)
	sitcp.write2byte(0xb4000210, 50)	# fast threshold: 50ch
	sitcp.write2byte(0xb4000216, 40)	# slow threshold: 40ch
	sitcp.write2byte(0xb400021a, 1)		# polarity: negative
	sitcp.write2byte(0xb400023a, 3)		# digital coarse gain: x8
	sitcp.write2byte(0xb400023c, 4095)	# digital fine gain: x0.5 (0.5 * 8103 - 2 -> 4095)
	sitcp.write2byte(0xb4000256, 0)		# analog pole zero: 0 ch
	sitcp.write2byte(0xb4000258, 200)	# analog fine gain: x1.176471
	sitcp.write2byte(0xb400020c, 0)		# fast pole zero: 0ch
	sitcp.write2byte(0xb400020e, 65)	# slow pole zero: 65ch

	# global register
	sitcp.write2byte(0xb400007a, 2)			# DAC monitor: CH1-slow
	sitcp.write2byte(0xb4000010, 0)			# mode: histogram
	sitcp.write6byte(0xb4000016, 300000000)	# measurement time: 3sec(10ns/digit)

	# filter reset
	sitcp.write2byte(0xb4000238, 0)
	sitcp.write2byte(0xb4000238, 1)
	sitcp.write2byte(0xb4000238, 0)

	# clear
	sitcp.write2byte(0xb4000040, 0)
	sitcp.write2byte(0xb4000040, 1)
	sitcp.write2byte(0xb4000040, 0)
	time.sleep(0.2)		# wait

	return


##########################################################
if __name__=='__main__':
	main(sys.argv)

